header{* Beispiele für die einfache Prädikatenlogik. *}
theory VSFOLEx
imports "../VSFOL"
begin

section{* Ohne Existenquantor *}

text{*
  Ein ganz einfaches Beispiel, um Isabelles Umgang mit Quantoren
  zu demonstrieren. 
  *}

lemma bsp0: "(\<forall> x. f x) \<longrightarrow> (\<forall> b. f b)"
 apply (rule impI)
 apply (rule allI)
 apply (drule allE) --{* Hier können wir nicht von Hand instantiieren, z.B. [where t="B"]*}
 apply (assumption) --{* ... zum Glück instantiiert Isabelle für uns. *}
 done

text{* Das erste Tafelbeispiel *}

lemma bsp1: "(\<forall> x. \<forall> y. f x y) \<longrightarrow> (\<forall> y. \<forall> x. f x y)"
 apply (rule impI)
 apply (rule allI)
 apply (rule allI)
 apply (drule allE)
 apply (drule allE)
 apply (assumption)
 done
 
text{* In diesem Beispiel elimieren wir erst die Allquantoren
 in den Prämissen. Dadurch entstehen freie Meta-Variablen, die nicht
 im Gültigkeitsbereich der Variablen sind, die durch die Elimination
 der Allquantoren in der Konklusion entstehen, und die Isabelle deshalb
 nicht instantieren kann.
 *}
lemma bsp1_falsch: "(\<forall> x. \<forall> y. f x y) \<longrightarrow> (\<forall> y. \<forall> x. f x y)"
 apply (rule impI)
 apply (drule allE)
 apply (drule allE) --{* Meta-Variablen t2, t4 *}
 apply (rule allI)
 apply (rule allI) 
 apply (assumption) -- {* t2, t4 können nicht zu x, y, instantiiert werden. *}
 oops
 
text{* Das zweite Tafelbeispiel. *}

lemma bsp2: "(\<forall> x. f x \<and> g x) \<longrightarrow> (\<forall> x. f x) \<and> (\<forall> x. g x)"
  apply (rule impI)
  apply (rule conjI)
  --{* Erster Zweig *}
  apply (rule allI)
  apply (drule allE)
  apply (drule conjE1)
  apply (assumption)
  --{* Zweiter Zweig *}
  apply (rule allI)
  apply (drule allE)
  apply (drule conjE2)
  apply (assumption)
  done

text{* Alternativ-Version *}

lemma bsp2a: "(\<forall> x. f x) \<and> (\<forall> x. g x) \<longrightarrow> (\<forall> x. f x \<and> g x)"
 apply (rule impI)
 apply (rule allI)
 apply (rule conjI)
 apply (rule allE [where P="f"])
 apply (drule conjE1)
 apply (assumption)
 apply (rule allE [where P="g"])
 apply (drule conjE2)
 apply (assumption)
 done


section {* Mit Existenzquantor *}

lemma bsp3: "(\<forall> x. u x \<longrightarrow> v) \<longrightarrow> ((\<exists> x. u x) \<longrightarrow> v)"
  apply (rule impI)
  apply (rule impI)
  apply (erule exE [where P="%x. u x"])
  apply (drule allE)
  apply (drule impE)
  apply (assumption)
  apply (assumption)
  done

end
  
