header {* Very Simple Propositional Logic II *}
theory VSPL
imports VSPL0
begin

text{*
  Hier wird eine Konstante \emph{definiert}, d.h. sie wird als
  abkürzende Schreibweise vereinbart:
  *}

definition
  Not    :: "o => o"                     ("\<not> _" [40] 40)
where
  "\<not>P    == P \<longrightarrow> \<bottom>"


text{*
  Die Einführungs- und Eliminationsregel für Not werden als
  Theoreme \emph{bewiesen}. Im wesentlichen handelt es sich 
  um die Einführungs- und Eliminationsregel für --> mit 
  geeigneter Instanziierung für P (die hier Isabelle für uns
  erledigt).
  *}
  
ML_val {* @{thm Not_def} *}
  
lemma notE: "[| \<not>P; P |] ==> \<bottom>"
  apply (unfold Not_def)
   --{* Mit unfold wird die linke Seite einer Definition durch 
        rechte Seite ersetzt; die Definition wird "aufgefaltet". *}
  apply (erule impE)
  apply (assumption)
  done

lemma notI: "(P ==> \<bottom>) ==> \<not>P"
  apply (unfold Not_def)
  apply (rule impI)
  apply (assumption)
  done

text{*
  Hier wird das Lemma Raa' definiert als das Lemma Raa, in dem alle Vorkommen
  von P \<longrightarrow> \<bottom> durch \<not>P ersetzt werden ("einfalten" der Konstantendefinition).
  *}

lemmas Raa' = Raa [folded Not_def]
   and notnot  = doubleneg [folded Not_def]
   and notnot2 = doubleneg2 [folded Not_def]

ML_val{* @{thm notnot2} *}
   
text{*
  Definition der Disjunktion, sowie die Einführungs- und Elemininationsregeln.
  *}

definition
  Or  :: "[o, o] => o"                (infixr "\<or>" 30)
where
  "P \<or> Q == \<not>(\<not>P \<and> \<not>Q)"

theorem disjI1: "P ==> P \<or> Q"
  apply (unfold Or_def)
  apply (rule notI)
  apply (rule notE [where P="P"])
  apply (erule conjE1) 
  apply (assumption)
  done

lemma disjI2: "Q ==> P \<or> Q"
  apply (unfold Or_def)
  apply (rule notI)
  apply (rule notE [where P="Q"])
  apply (erule conjE2) 
  apply (assumption)
  done

lemma disjE: "[| P \<or> Q; P==> R; Q ==> R |] ==> R"
  apply (unfold Or_def)
  apply (rule Raa')
  apply (rule notE [where P="\<not>P \<and> \<not>Q"])
  apply (assumption)
  apply (rule conjI)
  apply (rule notI)
  apply (rule notE [where P="R"])
  apply (assumption)
  apply (drule impI)
  apply (erule impE [where P="P"])
  apply (assumption)
  apply (rule notI)
  apply (rule notE [where P="R"])
  apply (assumption)
  apply (drule impI [where P="Q"])
  apply (rule impE [where P="Q"])
  apply (assumption, assumption)
  done

lemma tertiumNonDatur: "P \<or> \<not> P"
  apply (unfold Or_def)
  apply (rule notI)
  apply (rule notE [where P="\<not> P"])
  apply (rule conjE2, assumption)
  apply (rule conjE1, assumption) 
  done

text{*
  Definition der Äquivalenz (Biimplikation) mit seinen Regeln. 
  *}
definition
  Iff :: "[o, o] => o"               (infixr "\<longleftrightarrow>" 20)
where
  "P \<longleftrightarrow> Q == ((P \<longrightarrow> Q) \<and> (Q \<longrightarrow> P))"

lemma iffI: "[| P \<longrightarrow> Q; Q \<longrightarrow> P |] ==> P \<longleftrightarrow> Q"
  apply (unfold Iff_def)
  apply (rule conjI)
  apply (assumption)
  apply (assumption)
  done

lemma iffE1: "[| P \<longleftrightarrow> Q |] ==> (P \<longrightarrow> Q)"
  apply (unfold Iff_def)
  apply (erule conjE1)
  done

lemma iffE2: "[| P \<longleftrightarrow> Q |] ==> (Q \<longrightarrow> P)"
  apply (unfold Iff_def)
  apply (erule conjE2)
  done

lemma iff_notnot: "\<not>\<not> P \<longleftrightarrow> P"
  apply (rule iffI)
  apply (rule notnot)
  apply (rule notnot2)
  done

lemma imp_refl: "P \<longrightarrow> P"
  apply (rule impI)
  apply (assumption)
  done


lemma iff_refl: "P \<longleftrightarrow> P"
  apply (rule iffI)
  apply (rule imp_refl)
  apply (rule imp_refl)
  done

end
